from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal

from cStringIO import StringIO
import string

from Products.Plone2Pdf import skin_globals
from Products.Plone2Pdf.config import *
from Products.CMFActionIcons import *
from Acquisition import aq_base
from Products.PortalTransforms.libtransforms.utils import bin_search

skin_names = ('Plone2Pdf','Plone2Pdf_images')

def addAction(self,out):
    pa = getToolByName(self, 'portal_actions')
    pa.addAction('topdf',
                 name='create PDF file',
                 action='string:${object_url}/html2pdf_entry',
                 condition='python: object.portal_type in portal.pdfConvertTypes()',
                 permission='View',
                 category='document_actions')

def addActionIcons(self, out):
    ai = getToolByName(self, 'portal_actionicons')
    try:
        ai.getActionIcon('plone','topdf')
    except KeyError:
        ai.addActionIcon('plone', 'topdf', 'pdf_icon.gif', 'create PDF file')

def plone2Pdf_properties(self, out):
    # install Plone2Pdf global properties
    
    http_proxy= ''
    
    plone2pdf_properties = (
    ('http_proxy', 'string', http_proxy),
    ('use_pdf_layout_form', 'boolean', False),
    )
    if not hasattr(self.portal_properties, 'plone2pdf_properties'):
        self.portal_properties.addPropertySheet(
            'plone2pdf_properties', 'Plone2Pdf properties')

    props = self.portal_properties.plone2pdf_properties
    for prop, tp, val in plone2pdf_properties:
        if not props.hasProperty(prop):
            props._setProperty(prop, val, tp)

    print >> out, "Successfully installed Plone2Pdf properties."
        
def setupSkin(self, out):
    skinsTool = getToolByName(self, 'portal_skins')
    
    # Add directory views
    try:  
        addDirectoryViews(skinsTool, 'skins', skin_globals)
        out.write( "Added directory views to portal_skins.\n" )
    except:
        out.write( '*** Unable to add directory views to portal_skins.\n')

    # Go through the skin configurations and insert the skin
    skins = skinsTool.getSkinSelections()
    for skin in skins:
        path = skinsTool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        changed = 0
        for skin_name in skin_names:
            if skin_name not in path:
                try: 
                    path.insert(path.index('custom')+1, skin_name)
                    changed = 1
                except ValueError:
                    path.append(skin_name)
                    changed = 1

        if changed:        
            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinsTool.addSkinSelection(skin, path)
            out.write("Added %s to %s skin\n" % (', '.join(skin_names),skin))
        else:
            out.write("Skipping %s skin, %s already set up\n" % (skin, ', '.join(skin_names)))
            
#RSC checking for necessary products  
def checkRequiredProducts( self, out):
    installTool = getToolByName( self, 'portal_quickinstaller' )
    for product in REQUIRED_PRODUCTS:
        if not installTool.isProductInstalled( product ):
            raise RuntimeError, '%s should be installed prior to installing this product.' % product

def installTransform(self,out): 
    # unreg if necessary
    unregTransform(self,out)

    # check for pisa    
    pisabin = bin_search('pisa')
    transform_tool = getToolByName(self, 'portal_transforms')
    from Products.Plone2Pdf.html_to_pdf import register
    transform_tool.registerTransform(register())    
    #transform_tool.manage_addTransform('html_to_pdf', 'Products.Plone2Pdf.html_to_pdf')
        
def install(self):
    out=StringIO()

    checkRequiredProducts(self,out)
    setupSkin(self, out)
    addActionIcons(self, out)
    plone2Pdf_properties(self,out)
    addAction(self, out)
    installTransform(self,out)
    out.write('Installation completed.\n')
    return out.getvalue()

#uninstall methods
def unregTransform(self,out):
    transform_tool = getToolByName(self, 'portal_transforms')
    
    try:
        # Remove existing transform, if any
        transform_tool.unregisterTransform('html_to_pdf')
    except:
        pass
    
    try:
        # Remove existing transform, if any
        transform_tool.manage_delObjects('html_to_pdf')
    except:
        pass
    
    

def uninstall(self):
    out = StringIO()
    # remove external methods from old version
    portal=getToolByName(self,'portal_url').getPortalObject()
    
    for script in ('html2pdf', ):
        if hasattr(aq_base(portal), script):
            meth = getattr(aq_base(portal), script)
            if meth.meta_type=='External Method':
                out.write('Deleting External Method.\n')
                portal.manage_delObjects(ids=[script,])
                
    # fix bug : html2pdf not present anymore (Ext Method)
    portal.portal_quickinstaller.Plone2Pdf.portalobjects = []
    
    # unregister Transform
    unregTransform(self,out)
    
    return out.getvalue()
