"""
Uses pisa from http://www.htmltopdf.org to do its handy work

"""
from Products.PortalTransforms.interfaces import itransform
from Products.PortalTransforms.libtransforms.utils import bin_search, sansext
from Products.PortalTransforms.libtransforms.commandtransform import commandtransform

from os.path import join, abspath, dirname, basename
from os import system,sep
from Products.Plone2Pdf.config import DEBUG
from Products.Plone2Pdf.manipulation import handleImages, handleCSS, addLinkList
import logging

logger = logging.getLogger('Plone2Pdf')

class html_to_pdf(commandtransform):
    __implements__ = itransform
    
    __version__ = '2008-01-15.01'

    __name__ = "html_to_pdf"
    inputs   = ('text/html',)
    output  = 'application/pdf'
    output_encoding = 'utf-8'

    binaryName = "pisa"
    binaryArgs = ""
    useStdin = False

    def __init__(self):
        commandtransform.__init__(self, binary=self.binaryName)

    def convert(self, data, cache, **kwargs):

        input_data = data.viewPdf()

        tmpdir, fullname = self.initialize_tmpdir(input_data, filename='orig.html')

        input_data = handleCSS(data, input_data, tmpdir)
        input_data = handleImages(data, input_data, tmpdir)

        if kwargs.get('add_link_list',None):
           input_data = addLinkList(input_data)

        src = '%s%sfixed.html' % (tmpdir,sep)

        filedest = open(src , "wb").write(input_data)

        pdf = self.invokeCommand(tmpdir, src)

        self.cleanDir(tmpdir)

        cache.setData(pdf)

        return cache


    def invokeCommand(self, tmpdir, src):
        
        try:

           dest = '%s.pdf' % src
           
          
           pisa_opts={'sep' : sep,
    		      'tmpdir': tmpdir,
                      'binary': self.binary,
                      'src' : src,
                      'dest': dest,
                     }
          
           if not DEBUG:
              binCmd = '%(binary)s -q %(src)s %(dest)s 1>%(tmpdir)s%(sep)spisa.log 2>%(tmpdir)s%(sep)serror.log' % pisa_opts
           else:
              binCmd = '%(binary)s -w %(src)s %(dest)s 1>%(tmpdir)s%(sep)spisa.log 2>%(tmpdir)s%(sep)serror.log' % pisa_opts

           system(binCmd)

           self.debugMsg(tmpdir)

           outfile = open(dest, 'r')

           pdf = outfile.read()

           outfile.close()
        
        except:
            try:
               self.debugMsg(tmpdir)
               msg = open("%s/error.log" % tmpdir, 'r').read()
               return msg
            except:
               return ''

        return pdf


    def debugMsg(self,tmpdir):
        if DEBUG:
           try:
              error_msg=open("%s%serror.log" % (tmpdir,sep), 'r').read()
              logger.info(error_msg)
           except:
              logger.info('cannot read %s%serror.log' % (tmpdir,sep))
           try:
              pisa_msg=open("%s%spisa.log" % (tmpdir,sep), 'r').read()
              logger.info(pisa_msg)
           except:
              logger.info('cannot read %s%spisa.log' % (tmpdir,sep))        


def register():
    return html_to_pdf()
