##########################################################################
# SmartPrintNG - high-quality export of Plone content to
# PDF, RTF, ODT, WML and DOCX
#
# (C) 2007, ZOPYX Ltd & Co. KG, Tuebingen, Germany
##########################################################################

""" HTML manipulation methods """

import re
import os
import random
import htmllib
import formatter
from cStringIO import StringIO
from urlparse import urlparse, urlsplit
import urllib2



from PIL import Image
from BeautifulSoup import BeautifulSoup, NavigableString

from Products.CMFCore.utils import getToolByName 

def removeLinks(html):
    """ replace all links with the tag text """

    soup = BeautifulSoup(html)
    refs = soup.findAll('a')
    [ref.extract() for ref in refs]
    return  str(soup)

def removeImages(html):
    """ Remove all images """

    soup = BeautifulSoup(html)
    images = soup.findAll('img')
    [img.extract() for img in images]
    return  str(soup)


def checkHref(href):
    """ Return False for mailto|javascript or internal
        anchors or views.
    """

    if 'mailto:' in href or \
       'javascript:' in href or \
       href.startswith('#') or \
       href.startswith('@@'):
           return False
    return True
       

def getLinksInHtml(html):
    """ return all links inside a HTML fragment """

    soup = BeautifulSoup(html)
    hrefs = {}
    count = 0
    for anchor in soup.findAll('a'):

        # we need to capture internal anchors
        try:
            href = str(anchor['href'])
        except:
            continue 
        if not checkHref(href):
            continue

        if hrefs.has_key(href):
           count = hrefs[href]
        else:
           count += 1
           hrefs[href] = count
    links=map(lambda x: (x[1], x[0]),hrefs.iteritems())
    links.sort()
    return links


def _pcdataFromNode(s, lst=[]):
    """ recursive pcdata collector """

    if s.string is not None:
        lst.append(s.string)
    else:
        for n in s.contents:
            _pcdataFromNode(n, lst)
    return ' '.join(lst)


def enumerateLinksInHtml(html, links):

    soup = BeautifulSoup(html)

    count = 0
    hrefs = {}
    for anchor in soup.findAll('a'):

        # capture internal anchors
        try:
            href = str(anchor['href'])
        except:
             
            continue

        if not checkHref(href):
            continue

        anchor['class'] = 'enumerated-link'
        contents = anchor.contents

        if hrefs.has_key(href):
           count = hrefs[href]
        else:
           count += 1
           hrefs[href] = count

        contents.append(NavigableString(u'&nbsp;[%d]' % count))

        anchor.contents = contents

        
    return str(soup)
    

def addLinkList(html):

    links = getLinksInHtml(html)
    html = enumerateLinksInHtml(html, links)

    if links:                      
        pat = '<li>[%d] %s</li>'
        link_string = '\n'.join([pat % (i, link) for i,link in links])
        linkLst = '\n<div class="enumerated-links"><ol>%s</ol</div>' % link_string
        html = html + linkLst

    return html


def handleCSS(context, html, tempdir):

    def fixCSS(link):
        url = link['href'].encode('ascii')
        tp = urlparse(url)
        suffix = urlsplit(url)[2].split(".")[-1]

        if suffix.lower() not in ("css",):
           link.extract()
           return

        new_css = '%s/%s.css' % (tempdir, random.random())

        css_data = ''
        if tp[0] in ('http', 'https', 'ftp'):
           data = urlopen(context,url)
           if data:
               css_data = data.read()

        if css_data:
           tfile = open(new_css, "wb")
           tfile.write(str(css_data))
           tfile.close()
           link['href'] = new_css

        else:
           link.extract()
     
    soup = BeautifulSoup(html)

    for link in soup.findAll('link'):
       fixCSS(link)

    html = str(soup)
    return html    


def urlopen(context,url):
    pp = getToolByName(context, 'portal_properties')
    http_proxy = pp.plone2pdf_properties.getProperty('http_proxy',None)

    try:
       if http_proxy:
          proxy_handler = urllib2.ProxyHandler({'http': http_proxy})
	  opener = urllib2.build_opener(proxy_handler)    
          opener.add_handler(proxy_handler)
          data = opener.open(url)
       else:
          data = urllib2.urlopen(url)
    except:
          data = None

    return data


def handleImages(context, html, tempdir):

    mt = getToolByName(context, 'portal_membership')

    http_host = context.REQUEST.HTTP_HOST

    def fixImages(img):

        src = img['src'].encode('ascii')

        tp = urlparse(src)
        img_data = ''             

        same_host = http_host == tp[1]
#        if tp[0] in ('http', 'https', 'ftp') and not same_host:
        if tp[0] in ('http', 'https', 'ftp'):
            # appears like a remote image, download it
            url = src
            data = urlopen(context,url)
            if data:
               img_data = data.read()

        else:
            # likely a local image, try to traverse to get hold
            # of the image data directly
            img_path = src
            if img_path.startswith('./'): # FCKeditor 
               img_path = img_path[2:]

            if img_path.lower().startswith('resolveuid'): # FCKeditor 
               reference_tool = getToolByName(context, 'reference_catalog')
               spath = img_path.split('/')
	       uuid =  spath[1]
               img_obj = reference_tool.lookupObject(uuid)
               if spath[2:]:
                  img_obj = img_obj.unrestrictedTraverse('/'.join(spath[2:]), None)
               
               if img_obj:
                  if mt.checkPermission('View', img_obj):
                     try:
                        img_data = str(img_obj.data)
                     except AttributeError:
                         try:
                            img_data = str(img_obj._data) # FSImage
                         except AttributeError:
                            pass       
            else:
               img_obj = None
               for path in (img_path, tp[2]):        
                  img_obj = context.unrestrictedTraverse(path, None)
                  if img_obj:
                     if mt.checkPermission('View', img_obj):
                        try:
                            img_data = str(img_obj.data)
                            break
                        except AttributeError:
                            try:
                                img_data = str(img_obj._data) # FSImage
                            except AttributeError:
                                break

        if img_data:        
            pil_img= Image.open(StringIO(img_data))

            new_img = '%s/%s.%s' % (tempdir, random.random(),pil_img.format)

            tfile = open(new_img, "wb")
            tfile.write(str(img_data))
            tfile.close()

            img['src'] = new_img

        else:
            img.extract()

    soup = BeautifulSoup(html)
    for img in soup.findAll('img'):
        fixImages(img)
    html = str(soup)
    return html


def breakIntoPages(html, seperator='(h1|h2)'):

    breaker = re.compile('<%s' % seperator, re.I|re.M|re.S)

    div_start = '<div class="chapter sp-page">'
    div_start2 = '<div class="chapter">'
    div_end = '</div>'

    positions = []
    for mo in breaker.finditer(html):
        positions.append(mo.start())
    positions.append(len(html))

    parts = []
    len_positions = len(positions) - 1
    for i in range(len_positions):
        start = positions[i]
        end = positions[i+1]

        if i == len_positions - 1:
            parts.append(div_start2 + html[start: end].strip() + div_end)
        else:
            parts.append(div_start + html[start: end].strip() + div_end)

    return '\n'.join(parts)



available_manipulations = (
    {'label' : 'removeImages', 'method' : removeImages, 'description': 'Remove images'},
    {'label' : 'removeLinks', 'method' : removeLinks, 'description': 'Remove links'},
    {'label' : 'addLinkList', 'method' : addLinkList, 'description': 'Add link list'},
    {'label' : 'breakIntoPages', 'method' : breakIntoPages, 'description': 'Break into pages'},
)
